# flux-up
Upsample neutron simulation results from low to high group and spatial resolution using Convolutional Neural Network.


# Install and setup instructions

IMPORTANT: Install/Setup instructions assume this is being done in Linux. Install/setup tested in Ubuntu 20.04, and 22.04.

## OpenMC

OpenMC is needed to generate training data for the CNN. This guide assumes that the openmc-dev version on github is 0.14.1. Any other version might require an update to flux-up. To install OpenMC first need prerequisites:

```
sudo apt update
sudo apt install git
sudo apt install g++
sudo apt install cmake
sudo apt install libhdf5-dev
sudo apt install libpng-dev
cd $HOME
mkdir codes && cd codes
git clone --recurse-submodules https://github.com/openmc-dev/openmc.git
```
Then compile OpenMC:

```
cd openmc
mkdir build && cd build
cmake ..
make
```

the openmc binary will be in the build/bin directory.

## OpenMC Python API

This is needed to allow you to interact with OpenMC via Python, which is much easier than C++. The API is installed via pip. Using the default system installed pip (which is usually built off python2.7 for Mac and Ubuntu) is STRONGLY DISCOURAGED. If you wish to do this, do it at your own risk. The safest way is to first install conda and create a new conda environment specific for working with OpenMC and tensorflow. This README assumes you have some familiarity with conda.

1. Install miniconda
2. Create a new conda environment with a name of your choice, e.g. tensorflow-openmc-0.14.1, and python version 3.11. As of writing this guide, TF2 is not compatible with any higher version of python. Activate it:
   
```
conda create --name tensorflow-openmc-0.14.1 python=3.11
conda activate tensorflow-openmc-0.14.1
```
3. Install pip and numpy with conda, and install tensorflow2 and sklearn with pip:

```
conda install pip
conda install numpy
pip install tensorflow
pip install scikit-learn

```

The second-to-last line should install the GPU enabled version of TF2. To verify, enter:

```
python3 -c "import tensorflow as tf; print(tf.config.list_physical_devices('GPU'))"
```

It should return a bunch of warnings, look for the following line which will confirm success:

```
[PhysicalDevice(name='/physical_device:GPU:0', device_type='GPU')]
```

4. Install OpenMC Python API

```
conda activate tensorflow-openmc-0.14.1
cd $HOME/codes/openmc
pip install .
```

## OpenMC nuclear data libraries

1. Download ENDF/B-VIII.0 files from https://openmc.org/official-data-libraries/. They are quite large so put them somewhere sensible.
2. Unpack them with tar -xvf filename.tar.xz or whatever.

## BEAVRS

BEAVRS is a PWR benchmark from MIT. A full model for OpenMC can be found at https://github.com/mit-crpg/BEAVRS. We will use it to define our materials etc, it's convenient because of pre-written functions for doing this.

```
conda activate tensorflow-openmc-0.14.1
cd ~/codes
git clone https://github.com/mit-crpg/BEAVRS.git
cd BEAVRS/models/openmc
pip install .
```



## Other Dependencies

flux-up also requires the iapws and scikit-optimize libraries:

```
conda activate tensorflow-openmc-0.14.1
pip install iapws
pip install scikit-optimize
pip install pydot
pip install graphviz
conda install pytables
```
Also, scikit-optimize seems to be a dead project, so it's no longer being updated. Once it's installed, some edits must be made:

```
code ~/miniconda3/envs/tensorflow-openmc-0.14.1/lib/python3.11/site-packages/skopt/space/transformers.py
```
change all references of `np.int` to `int`


## Quickstart

Open up example-openmc-inputs/assembly.py and change the lines:

```
openmc.Materials.cross_sections = '/home/andrew/nuclear/mcnpx-shared/openmc-nuclear-data/endfb80_hdf5/cross_sections.xml'
openmcExecutable = '/home/andrew/codes/openmc/build/bin/openmc'
```

to point to where your cross_sections.xml and openmc binary are installed. Then activate your conda environment and in the command line type:

```
python example-openmc-inputs/assembly.py --run
```

this will create the XML input files, run openMC and produce output files. You can visualize the tallies by using the code openmc-plot-mesh-tally that comes with openmc:

```
python /path/to/openmc/scripts/openmc-plot-mesh-tally
```

select the statepoint file that openmc creates and use the GUI to see the tally.

## References/Useful papers

Adversarial super-resolution of climatological wind and solar data - https://www.pnas.org/doi/full/10.1073/pnas.1918964117