from src.OpenMCWrapper import OpenMCWrapper
import beavrs.builder
import beavrs.constants as c
import openmc
from src.tallies import meshTally

class BeavrsAssemblyOpenMCWrapper(OpenMCWrapper):
    
    def __init__(self, casesDataFrame,NgroupsIn=15,NgroupsOut=30,resolutionIn=[32,32], resolutionOut=[128,128]):
        super().__init__(casesDataFrame,NgroupsIn,NgroupsOut,resolutionIn, resolutionOut)

    def generateModel(self,idx):
        assembly = self.casesDataFrame.iloc[idx]['lattice_object']
        b = beavrs.builder.BEAVRS()
        # box, infinite in z
        #assemblyPitch = 21.42 #17*c.pinPitch # for now use the same pitch as was used to train the CNN
        assemblyPitch = 17*c.pinPitch 
        lattice_sides = openmc.model.rectangular_prism(assemblyPitch, assemblyPitch,
                                                   boundary_type='reflective')
        # bounding planes on lower and upper z (1cm thickness)
        #min_z = openmc.ZPlane(z0=-0.5, boundary_type='reflective')
        #max_z = openmc.ZPlane(z0=0.5, boundary_type='reflective')
        min_z = openmc.ZPlane(z0=c.struct_LowestExtent, boundary_type='vacuum')
        max_z = openmc.ZPlane(z0=c.struct_HighestExtent, boundary_type='vacuum')
        # Create a "root" cell filled by the fuel assembly
        root_cell = openmc.Cell(name='root cell',
                        fill=assembly,
                        region=lattice_sides & +min_z & -max_z
                       )
        # Create a "root" universe with ID=0 and add the "root" cell to it
        root_univ = openmc.Universe(name='root universe', cells=[root_cell])               
        # Create an OpenMC Geometry around root Universe
        sub_geometry = openmc.Geometry(root_univ)
        # create OpenMC model, attach geometry to it
        model = openmc.model.Model()
        model.geometry = sub_geometry
        model.settings.batches = 150
        model.settings.inactive = 50
        model.settings.particles = 100000
        # Use a bounding box to define the starting source distribution
        lower_left = [-17*c.pinPitch/2, -17*c.pinPitch/2, c.fuel_ActiveFuel_bot]
        upper_right = [+17*c.pinPitch/2, +17*c.pinPitch/2, c.fuel_ActiveFuel_top]
        model.settings.source = openmc.source.Source(
            openmc.stats.Box(lower_left, upper_right, only_fissionable=True))
        #model.settings.source = openmc.Source(space=openmc.stats.Box(
        #    (-assemblyPitch/2, -assemblyPitch/2, -0.5),
        #    (assemblyPitch/2, assemblyPitch/2, 0.5),
        #    only_fissionable=True
        #))
        #model.assemblyPitch = assemblyPitch
        model.assemblyPitch = 21.42
        return model

    def createPlots(self, model, idx=0):
        print("Creating plots...")
        plot = openmc.Plot()
        plot.filename = 'geometry_plot_{:d}'.format(idx)
        plot.width = (model.assemblyPitch, model.assemblyPitch)
        plot.origin = [0, 0, 200]
        plot.pixels = (800,800)
        plot.color_by = 'material'
        materials = model.geometry.get_all_materials()
        plot.colors = {}
        for materialIDNum in materials:
            material = materials[materialIDNum]
            plot.colors[materialIDNum] = 'black'
            if 'Water SPN' in material.name:
                plot.colors[materialIDNum] = 'cyan'    
            if 'Fuel' in material.name:
                plot.colors[materialIDNum] = 'red'
            if 'Borated Water' in material.name:
                plot.colors[materialIDNum] = 'blue'
            if 'B4C' in material.name:
                plot.colors[materialIDNum] = 'gray'
            if 'Ag-In-Cd' in material.name:
                plot.colors[materialIDNum] = 'yellow'    

        plots = openmc.Plots([plot])
        plots.export_to_xml()
        openmc.plot_geometry(openmc_exec=self.openMCExecutable)