import tensorflow as tf

class CompositeModel(object):
    def __init__(self, modelFunLambda, predictFun=None, num_models=5):
        self.num_models = num_models
        if(modelFunLambda != None):
            self.models = [modelFunLambda() for _ in range(num_models)]
        self.predictFun = predictFun

    def compile(self,optimizer='adam',loss='mae',metrics='mae'):
        
        for model in self.models:
            model.compile(optimizer,loss,metrics=metrics)    
    
    def fit(self,x,y,validation_split=0.2,epochs=None,batch_size=None,verbose=False,shuffle=True,validation_data=None):
        for model in self.models:
            model.fit(x=x,y=y,validation_split=validation_split,epochs=epochs,batch_size=batch_size,verbose=verbose,shuffle=shuffle,validation_data=validation_data)

    def save(self,rootDir):
        counter = 0
        for model in self.models:
            model.save(rootDir + '/model' + str(counter) + '/')
            counter += 1 

    def load_weights(self,rootDir):
        counter = 0
        for model in self.models:
            model.load_weights(rootDir + '/model' + str(counter) + '/')  
            counter += 1

    def first(self):
        return self.models[0]   

    @classmethod
    def load(cls, rootDir,predictFun=None,custom_objects=None,num_models=1):
        em = cls(None,predictFun)
        em.models = [tf.keras.models.load_model(rootDir + '/model' + str(i) + '/',custom_objects=custom_objects, compile=False) for i in range(num_models)]
        return em