import tensorflow as tf
import src
import numpy as np
from src.CompositeModel import CompositeModel

class EnsembleModel(CompositeModel):
        
    def predict(self,X,NDropoutSamples=1):
        useDropout=False
        if(NDropoutSamples > 1):
            useDropout = True

        if(self.predictFun == None):
            if(useDropout):
                preds = tf.stack([model(X,training=True) for _ in range(NDropoutSamples) for model in self.models],axis=0)
            else:
                preds = tf.stack([model.predict(X) for model in self.models],axis=0)
        else:
            preds = tf.stack([self.predictFun(model, X) for model in self.models],axis=0)
        mean = tf.math.reduce_mean(preds,axis=0)
        std  = tf.math.reduce_std(preds,axis=0)
        return mean,std
    
    
    def predict_and_postprocess(self,X,scalingValue,NDropoutSamples=1):
        """ Predict, unscale and reshape for compatibility in one convenient method
         
        Parameters
        ----------
        X:              [Nsamples x NgroupsIn x Nx x Ny] input data
        scalingValue:   [NgroupsIn] array of scaling values, one per energy group

        Returns
        ----------
        y:              [Nsamples x NgroupsOut x Nx x Ny] unscaled output data, representing mean of ensemble preds
        y_scaled:       [Nsamples x NgroupsOut x Nx x Ny] scaled output data, representing mean of ensemble preds
        y_stdev:              [Nsamples x NgroupsOut x Nx x Ny] unscaled output data, representing stdev of ensemble preds
        y_stdev_scaled:       [Nsamples x NgroupsOut x Nx x Ny] scaled output data, representing stdev of ensemble preds
        """
        y_scaled,y_stdev_scaled = self.predict(X,NDropoutSamples)
        y_scaled = src.utilities.unmoveaxis(y_scaled)
        y_stdev_scaled = src.utilities.unmoveaxis(y_stdev_scaled)
        y = src.normalization.normalize_by_group_only(np.copy(y_scaled),False,np.divide(1,scalingValue))
        y_stdev = src.normalization.normalize_by_group_only(np.copy(y_stdev_scaled),False,np.divide(1,scalingValue))
        return y,y_scaled,y_stdev,y_stdev_scaled

  