from src.CompositeModel import CompositeModel
from src.EnsembleModel import EnsembleModel
import tensorflow as tf
import numpy as np

'''
Class specifically for making predictions group-by-group, in an 8 group data set.

'''

class GroupModel(EnsembleModel):
    def __init__(self,modelFunLambda,predictFun=None):
        super().__init__(modelFunLambda, predictFun,num_models=8)
        self.validation_split = 0.2
        self.epochs = 50
        self.batch_size = 5
        self.optimizer_boundaries = np.repeat([20000], self.num_models,axis=0)
        self.optimizer_values = np.repeat([[5e-4,1e-4]], self.num_models,axis=0)
        self.loss = np.repeat('mae',self.num_models,axis=0).tolist()
    
    def fit(self,x,y):
        for i in range(self.num_models):
            self.fit_single_model(x,y,i)

    def fit_single_model(self,x,y,index=0):
        self.models[index].fit(x=x,y=y[:,:,:,index],validation_split=0.2,epochs=50,batch_size=5,verbose=True,shuffle=True)
    
    def compile(self):
        
        metrics=[tf.keras.metrics.MeanSquaredLogarithmicError(), tf.keras.metrics.MeanAbsolutePercentageError(),'mae','mse']
        for i in range(self.num_models):
            optim_edsr = tf.keras.optimizers.Adam(
                learning_rate=tf.keras.optimizers.schedules.PiecewiseConstantDecay(
                    boundaries=[self.optimizer_boundaries[i]], values=self.optimizer_values[i].tolist()))
            #self.models[i].compile(optimizer=tf.keras.optimizers.Adam(learning_rate=1e-4),loss=loss, metrics=metrics)
            self.models[i].compile(optimizer=optim_edsr, loss=self.loss[i], metrics=metrics)
           

    def predict_(self,X):
        preds = tf.concat([model.predict(X) for model in self.models],axis=-1)
        return preds

    def predict(self,X,NDropoutSamples=1):
        '''For compatibility with EnsembleModel
        '''
        preds = self.predict_(X)
        stdev = np.empty(shape=np.shape(preds))
        return preds, stdev


  