import pickle
import numpy as np

class OpenMCDepletion:


    def __init__(self,openMCRunDir=None):
        if(openMCRunDir != None):
            with open(openMCRunDir + '/depleted_materials.pickle', 'rb') as f:
                self.depletedMaterialsList = pickle.load(f)

    def NBUSteps(self):
        return len(self.depletedMaterialsList)    


    def get_material_name(self, matNameStr, burnupIdx=None):
        if burnupIdx != None:
            materials = self.depletedMaterialsList[burnupIdx]
            for material in materials:
                if(material.name == matNameStr):
                    return material
        material_vs_bu = []
        for i in range(0,self.NBUSteps()):
            for material in self.depletedMaterialsList[i]:
                if(material.name == matNameStr): 
                    material_vs_bu.append(material)
        return material_vs_bu

    def summary(self):
        for i in range(0,self.NBUSteps()):
            matsi = self.depletedMaterialsList[i]
            print('Burnup step: %f\n',i)
            [print(material.id, material.name) for material in matsi]

    def merge_depletion(matNameDirDict, existingMaterials=None):
        outputDictionary = {}
        for matName,directory in matNameDirDict.items():
            omcd = OpenMCDepletion(directory)
            outputDictionary[matName] = omcd.get_material_name(matName)
        if existingMaterials != None:
            allExistingIDs = np.array([material.id for material in existingMaterials.values()])
            newID = np.max(allExistingIDs) + 1
            for matName, matList in outputDictionary.items():
                for mat in matList:
                    mat.id = newID
                    newID += 1
        return outputDictionary