from tensorflow.keras import layers, Model
import tensorflow as tf
import numpy as np

def simple3DCNN(input_shape,output_shape,input_kernel):
    output_kernel = np.asarray(output_shape[:]) - np.asarray(input_shape[:-1]) + input_kernel
    model = tf.keras.Sequential([
    #layers.Input(shape=input_shape,name='input_layer'),
    #layers.Conv3D(filters=3,kernel_size=[2,2,2],activation='relu',name='convolution_layer'),
        layers.Conv3D(input_shape=input_shape, filters=1,kernel_size=input_kernel,activation='elu',name='convolution_layer'),
        layers.Conv3DTranspose(filters=1,kernel_size=output_kernel,activation='elu'),
    #layers.Conv3D(,name='output_layer'),
    ])
    return model


def ResBlock(inputs,Nfilters=64,dropoutRate=0,activations=['relu',None]):
    x = layers.Conv2D(Nfilters, 3, padding="same", activation=activations[0])(inputs)
    x = layers.Dropout(rate=dropoutRate)(x)
    x = layers.Conv2D(Nfilters, 3, padding="same",activation=activations[1])(x)
    x = layers.Dropout(rate=dropoutRate)(x)
    x = layers.Add()([inputs, x])
    return x    


def Upsampling(inputs, factor=2, **kwargs):
    """ Upsample an NxNxM array by a factor of 2 in the first 2 dimensions (= 2Nx2NxM).
    Call this function twice to upsample from 2x to 4x, and call 3 times to upsample by x8 and so on.
    This function only upsamples spatially. The last dimension (filters or channels) remains the same.

    Parameters
    ----------
    inputs: a tf.keras.layer
    factor: scalar, minimum of 2
        The function multiplies the number of channels by factor^2.
    **kwargs: extra arguments to layers.Conv2D

    Returns
    -------
    x: a tf.nn layer.

    """
    x = layers.Conv2D(64 * (factor ** 2), 3, padding="same", **kwargs)(inputs)
    x = tf.nn.depth_to_space(x, block_size=factor)
    #x = layers.Conv2D(64 * (factor ** 2), 3, padding="same", **kwargs)(x)
    #x = tf.nn.depth_to_space(x, block_size=factor)
    return x

def resnet_basic(Nfilters=64,kernelSize=3,NresBlocks=3,inputGroups=8,outputGroups=16,withIdentity=False,dropoutRate=0,activations=[None,None,None],resBlockActivations=['relu',None]):
    inputLayer = layers.Input(shape=(None,None,inputGroups))
    x = x_new = layers.Conv2D(Nfilters,kernelSize,padding="same",activation=activations[0])(inputLayer)
    # ResBlocks
    for _ in range(NresBlocks):
        x_new = ResBlock(x_new,Nfilters=Nfilters,dropoutRate=dropoutRate,activations=resBlockActivations)
    x_new = layers.Conv2D(Nfilters, kernelSize, padding="same",activation=activations[1])(x_new)
    x = layers.Add()([x, x_new])
    # upsampling block - each call to Upsampling will upsample by a factor of 2 in each direction.
    x = Upsampling(x)
    # increase channels (groups) to 16
    x = layers.Conv2D(outputGroups,3,padding="same",activation=activations[2])(x)
    # instantiate network
    if(withIdentity):
        identityLayerInput = layers.Input(shape=(None,None,outputGroups)) 
        identityLayerOutput = layers.Layer()(identityLayerInput) 
        x = layers.concatenate([x,identityLayerOutput])
        model = Model(inputs=[inputLayer,identityLayerInput],outputs=[x])
    else:
        model = Model(inputLayer,x)
    model.summary()
    #y_test = model.predict(moveaxis(X_val[0:1,:,:,:]))
    loss = 'mse' # works well
    from tensorflow.keras.utils import plot_model
    plot_model(model,to_file='model.png',show_shapes=True)
    return model

def resnet_basic_denoise(Nfilters=64,kernelSize=3,NresBlocks=3,inputGroups=8,outputGroups=16,withIdentity=False,dropoutRate=0):
    inputLayer = layers.Input(shape=(None,None,inputGroups))
    x = x_new = layers.Conv2D(Nfilters,kernelSize,padding="same")(inputLayer)
   
    # ResBlocks
    for _ in range(NresBlocks):
        x_new = ResBlock(x_new,Nfilters=Nfilters,dropoutRate=dropoutRate)
    x_new = layers.Conv2D(Nfilters, kernelSize, padding="same")(x_new)
    x = layers.Add()([x, x_new])
    # increase channels (groups) to 16
    x = layers.Conv2D(outputGroups,3,padding="same")(x)
    x = layers.Add()([inputLayer, x])
    # instantiate network
    if(withIdentity):
        identityLayerInput = layers.Input(shape=(None,None,outputGroups)) 
        identityLayerOutput = layers.Layer()(identityLayerInput) 
        x = layers.concatenate([x,identityLayerOutput])
        model = Model(inputs=[inputLayer,identityLayerInput],outputs=[x])
    else:
        model = Model(inputLayer,x)
    
    model.summary()
    #y_test = model.predict(moveaxis(X_val[0:1,:,:,:]))
    loss = 'mse' # works well
    from tensorflow.keras.utils import plot_model
    plot_model(model,to_file='model.png',show_shapes=True)
    return model 

