import numpy as np
from .context import src
import src.normalization as nz


def test_normalize_per_sample():
    np.random.seed(12345)
    #samples = np.random.rand(2,8,64,64) # generate 2 samples, 64x64 spatial and 8 energy bins
    samples = np.random.rand(2,1,2,2) # generate 2 samples, 2x2 spatial and 1 energy bin
    samples_normalized = nz.normalize_per_sample(np.copy(samples)) # need to send copy (pass by reference)
    np.testing.assert_allclose(samples_normalized.ravel(),[1., 0.34032926, 0.19784383, 0.22004813, 0.58861222, 0.6174554 , 1., 0.67720815],1e-7)
    samples_normalized,scalingFactors = nz.normalize_per_sample(np.copy(samples),True) # need to send copy (pass by reference)
    np.testing.assert_allclose([np.max(samples[0,:,:,:].ravel()), np.max(samples[1,:,:,:].ravel())],scalingFactors,1e-7)    
    samples_unnormalized = np.concatenate([np.multiply(samples_normalized[0,:,:,:].ravel(),scalingFactors[0]),
                            np.multiply(samples_normalized[1,:,:,:].ravel(),scalingFactors[1])])
    np.testing.assert_allclose(samples_unnormalized,samples.ravel() ,1e-7)    

def test_normalize_by_group():
    np.random.seed(12345)
    #samples = np.random.rand(2,8,64,64) # generate 2 samples, 64x64 spatial and 8 energy bins
    samples = np.random.rand(2,1,2,2) # generate 2 samples, 2x2 spatial and 1 energy bin
    samples_normalized,scalingFactors = nz.normalize_by_group(np.copy(samples),True) # need to send copy (pass by reference)
    np.testing.assert_allclose(samples_normalized.ravel(),[1., 0.34032926, 0.19784383, 0.22004813, 0.58861222, 0.6174554 , 1., 0.67720815],1e-7)
    np.testing.assert_allclose([np.max(samples[0,:,:,:].ravel()), np.max(samples[1,:,:,:].ravel())],scalingFactors.ravel(),1e-7) 
    
    samples = np.random.rand(2,8,64,64) # generate 2 samples, 64x64 spatial and 8 energy bins
    samples_normalized,scalingFactors = nz.normalize_by_group(np.copy(samples),True) # need to send copy (pass by reference)
    samples_unnormalized = np.concatenate([np.multiply(samples_normalized[0,:,:,:].reshape(8,64*64),scalingFactors[0,:].reshape(8,1)),
                            np.multiply(samples_normalized[1,:,:,:].reshape(8,64*64),scalingFactors[1,:].reshape(8,1))])
    np.testing.assert_allclose(samples_unnormalized.ravel(),samples.ravel() ,1e-7)