from .context import src
import src.utilities as ut
import os
import numpy as np

def test_merge_tallies_simple():
    homeDir = os.path.expanduser("~")
    datapath = [os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0005"),  # 50 samples
                os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0006")]  # 150 samples
    loFiTallies,hiFiTallies = ut.mergeTallies(datapath)
    assert np.shape(loFiTallies) == (200,6,64,64)
    assert np.shape(hiFiTallies) == (200,6,128,128)

def test_merge_tallies_uncertainty():
    homeDir = os.path.expanduser("~")
    datapath = [os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0005"),  # 50 samples
                os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0006")]  # 150 samples
    loFiTallies,hiFiTallies,loFiRelErr, hiFiRelErr, loFiStd, hiFiStd = ut.mergeTallies(datapath,True)
    assert np.shape(loFiTallies) == (200,6,64,64)
    assert np.shape(hiFiTallies) == (200,6,128,128)
    assert np.shape(loFiRelErr) == (200,6,64,64)
    assert np.shape(hiFiRelErr) == (200,6,128,128)
    assert np.shape(loFiStd) == (200,6,64,64)
    assert np.shape(hiFiStd) == (200,6,128,128)

def test_merge_tallies_flatten():
    homeDir = os.path.expanduser("~")
    datapath = [os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0005"),  # 50 samples
                os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0006")]  # 150 samples
    loFiTallies,hiFiTallies,loFiRelErr, hiFiRelErr, loFiStd, hiFiStd = ut.mergeTallies(datapath,True,True)
    assert np.shape(loFiTallies) == (200,64,64)
    assert np.shape(hiFiTallies) == (200,128,128)
    assert np.shape(loFiRelErr) == (200,64,64)
    assert np.shape(hiFiRelErr) == (200,128,128)
    assert np.shape(loFiStd) == (200,64,64)
    assert np.shape(hiFiStd) == (200,128,128)    

def test_noisify_tallies_normal_01():
    homeDir = os.path.expanduser("~")
    datapath = [os.path.join(homeDir,"nuclear/data/flux-up/2022-CRP-Project-Data/OpenMC-Runs/0008")]
    X_test,y_test, loFiRelErr, hiFiRelErr, loFiStd, hiFiStd = src.utilities.mergeTallies(datapath,includeUncertainty=True,flatten=False)
    X_test_noisy_1std = ut.noisify_tallies_normal(X_test,loFiStd,stdFac=1.0,seed=12345)
    X_test_noisy_2std = ut.noisify_tallies_normal(X_test,loFiStd,stdFac=2.0,seed=12345)
    groupNum = 3
    X_test_noisy_1std_flat = X_test_noisy_1std[:,groupNum,:,:].ravel()
    X_test_noisy_2std_flat = X_test_noisy_2std[:,groupNum,:,:].ravel()
    X_test_flat = X_test[:,groupNum,:,:].ravel()
    """ Debug: Plotting
    import matplotlib.pyplot as plt
    plt.hist(X_test_noisy_2std_flat-X_test_flat,100)
    plt.hist(X_test_noisy_1std_flat-X_test_flat,100)
    """
    #assert (X_test_noisy == X_test).all()
    assert np.std(X_test_noisy_2std_flat-X_test_flat) == 1.1917308315358788e-05
    assert np.std(X_test_noisy_1std_flat-X_test_flat) == 5.958654157679394e-06

def test_augment_tallies_01():
    testArr = np.reshape(np.array([[1, 2],[3, 4]]),(1,1,2,2))
    np.rot90(testArr,k=-1,axes=[2,3])
    augmentedArr = ut.augment_tallies_8fold(testArr)
    assert np.array_equal(augmentedArr[0,0,:,:], testArr[0,0,:,:]) # original
    assert np.array_equal(augmentedArr[1,0,:,:], np.array([[2, 1],
                                                            [4,3]])) # reflected
    assert np.array_equal(augmentedArr[2,0,:,:], np.array([[3, 1],
                                                            [4,2]])) # 90 deg rot
    assert np.array_equal(augmentedArr[3,0,:,:], np.array([[4, 3],
                                                            [2,1]])) # 180 deg rot
    assert np.array_equal(augmentedArr[4,0,:,:], np.array([[2, 4],
                                                            [1,3]])) # 270 deg rot
    assert np.array_equal(augmentedArr[5,0,:,:], np.array([[4, 2],
                                                            [3,1]])) # 90 deg rot, reflected
    assert np.array_equal(augmentedArr[6,0,:,:], np.array([[3, 4],
                                                            [1,2]])) # 180 deg rot, reflected
    assert np.array_equal(augmentedArr[7,0,:,:], np.array([[1, 3],
                                                            [2,4]])) # 270 deg rot, reflected
# should get 3x3 subsamples of shape 2x2
def test_subsample_sliding_window_01():
    testArr = np.reshape(np.linspace(1,7*7,7*7),(1,1,7,7))
    window = (2,2)
    stride = 2
    subsamples = ut.subsample_sliding_window(testArr,window,stride)
    assert np.shape(subsamples) == (9,1,2,2)

def test_subsample_sliding_window_02():
    testArr = np.reshape(np.linspace(1,2*2*7*7,2*2*7*7),(2,2,7,7))
    window = (2,2)
    stride = 2
    subsamples = ut.subsample_sliding_window(testArr,window,stride)
    assert np.shape(subsamples) == (18,2,2,2)

def test_subsample_sliding_window_03():
    testArr = np.reshape(np.linspace(1,2*3*7*7,2*3*7*7),(2,3,7,7))
    window = (2,2)
    stride = 2
    subsamples = ut.subsample_sliding_window(testArr,window,stride)
    assert np.shape(subsamples) == (18,3,2,2)



